"""
Network exercise part 2:  active model cell driven by noisy spike stream
"""

from neuron import h, gui # during development
import numpy as np
# for matplotlib's hist():
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt
# see documentation of matplotlib.pyplot.hist at matplotlib.org

NRUNS = 10 # number of simulations to execute before generating histogram

##### specification of model cell

from hhcell import Cell
cell = Cell()

PRESTART = 25 # ms so hh can settle
WEIGHT = 1.57e-5 # elicits 1 mV epsp in model with hh

##### specification of instrumentation

# synaptic mechanism
syn = h.ExpSyn(cell.soma(0.5), sec=cell.soma)
syn.tau = 3
syn.e = 0

# presynaptic spike source

pre = h.NetStim()
pre.start = PRESTART
pre.interval = 1
pre.number = 1e9
pre.noise = 1 # pseudorandom ISI governed by negexp distribution

nc = h.NetCon(pre, syn)
nc.delay = 1
nc.weight[0] = WEIGHT

# user interface for test runs
h.load_file("rig.ses") # RunControl, plots of cell.soma(0.5).v, syn.g

# recording spike times
stimes = h.Vector() # to capture times of elicited spikes
stnc = h.NetCon(cell.soma(0.5)._ref_v, None, sec = cell.soma)
stnc.threshold = 0
stnc.record(stimes)

# simulation flow control and post-run analysis
h.tstop = pre.start + 1000
h.tstop_changed() # update t axes of NEURON graphs

def onerun():
  """ do one run, return # spikes """
  h.run()
  return stimes.size()

def batch(n):
  """ do n runs, return list of # spikes """
  counts = []
  for i in range(n):
    counts.append(onerun())
  return counts

nspikes = batch(NRUNS)

# plotting a histogram
# print(nspikes) # during development for validation
num_bins = max(nspikes) - min(nspikes) + 1
# center each bin on a whole number of spikes
n, bins, patches = plt.hist(nspikes, num_bins, range=(min(nspikes)-0.5, max(nspikes)+0.5))
print('bin boundaries', bins)
print('bin counts', n)
plt.show()
